
/**
 * @file g9006_access.c
 * @author T-ICHIKAWA
 * @brief G9006アクセス用関数
 * @version 0.1
 * @date 2023-04-27
 * 
 * @copyright NPM (c) 2023
 * 
 */

#include "stdafx.h"
#include "g9006_access.h"
#include "spi_module.pio.h"              /* SPIモジュール */

/**
 * @brief SPIアクセス用モジュール
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param write_data SPI書き込みデータ
 * @return uint32_t SPI読み出しデータ
 */
uint32_t spi_access_module(PIO pio, uint sm, uint32_t write_data){

    while(pio_sm_is_tx_fifo_empty(pio,sm) == false);                 /* PIOのTX_FIFOが空でないなら無限待機 */
	pio_sm_put_blocking(pio, sm, write_data);                        /* PIOのTX_FIFOに書き込み */

    while(pio_sm_is_rx_fifo_empty(pio,sm) == true);                  /* PIOのRX_FIFOが空（読み出しデータが入っていない)時は無限待機 */
    uint32_t read_data = pio_sm_get_blocking(pio, sm);               /* RX_FIFOからデータを読み出す */

    return read_data;
}

/**
 * @brief SPI用ステートマシンの初期化
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param spi_mosi MOSI端子を指定
 * @param spi_mosi_num MOSI端子の本数を指定
 * @param spi_miso MISO端子を指定
 * @param spi_miso_num MISO端子の本数を指定
 * @param spi_side_base SCK,CS端子のベースピンを指定
 * @param spi_side_num SCK,CS端子合計の本数を指定
 * @param spi_sck_frq SCKの動作周波数を指定([10]を指定した場合、125MHzの10分周でステートマシンの動作周波数は12.5MHz,SCKの周波数は12.5MHz/4=3.125MHz)
 */
void spi_init( PIO pio, uint sm, uint spi_mosi, uint spi_mosi_num, uint spi_miso, uint spi_miso_num, uint spi_side_base, uint spi_side_num, float spi_sck_frq ){
	
    /* 指定したPIOのインストラクションメモリにプログラムをロードするプログラムの先頭アドレス(オフセット)が戻り値として返ってくる */
	uint offset = pio_add_program( pio, &spi_module_program );

    /* SPIモジュール部のステートマシンを初期化する */
	spi_module_init( pio, sm, offset, spi_mosi, spi_mosi_num, spi_miso, spi_miso_num, spi_side_base, spi_side_num, spi_sck_frq );
}

/**
 * @brief G9006_アドレスあり書き込み制御
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param comtype 制御コマンドタイプ選択(取説5.2.2.3タイプ選択に記載あり)
 * @param address 書き込みアドレス指定
 * @param data 書き込みデータ指定
 */
void write_reg_G9006(PIO pio, uint sm, uint32_t comtype, uint32_t address, uint32_t data){
    
    /* SPI書き込みデータをまとめる */
    uint32_t write_data = (comtype << 24) | (address << 16) | ((data << 8) & 0xFF00) | ((data >> 8) & 0x00FF);
    /* SPIで書き込み,読み出し(下位バイトから書き込み) */
    uint32_t read_data = spi_access_module(pio, sm, write_data);
}

/**
 * @brief G9006_アドレスあり読み出し制御
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param comtype 制御コマンドタイプ選択(取説5.2.2.3タイプ選択に記載あり)
 * @param address 読み出しアドレス指定
 * @return uint32_t 読み出しデータ戻り値
 */
uint32_t read_reg_G9006(PIO pio, uint sm, uint32_t comtype, uint32_t address){
    
    uint32_t write_data = (comtype << 24) | (address << 16) | 0x0000;      /* SPI書き込みデータをまとめる */
    uint32_t read_data = spi_access_module(pio, sm, write_data);           /* SPIで書き込み,読み出し */
    printf("%08X\n", read_data);                                           /* USBのUARTで送信する */

    read_data = read_data & 0x0000FFFF;
    return read_data;
}

/**
 * @brief G9006_アドレスなし読み出し制御
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param comtype 制御コマンドタイプ選択(取説5.2.2.3タイプ選択に記載あり)
 */
void read_str_G9006(PIO pio, uint sm, uint32_t comtype){
    
    uint32_t write_data = comtype << 24;                                  /* SPI書き込みデータをまとめる */
    uint32_t read_data = spi_access_module(pio, sm, write_data);          /* SPIで書き込み,読み出し */

    printf("%08X\n", read_data);                                          /* USBのUARTで送信する */
}

/**
 * @brief G9006_アドレスなし書き込み制御
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param comtype 制御コマンドタイプ選択(取説5.2.2.3タイプ選択に記載あり)
 * @param opecom 操作コマンド選択(取説5.2.2.3.3操作コマンド実行に記載あり)
 */
void write_opecom_G9006(PIO pio, uint sm, uint32_t comtype, uint32_t opecom){
    
    uint32_t write_data = (comtype << 24) | (opecom << 16) | 0x0000;      /* SPI書き込みデータをまとめる */
    uint32_t read_data = spi_access_module(pio, sm, write_data);          /* SPIで書き込み,読み出し */
}