
/**
 * @file PCL6046_G9006_demo.c
 * @author NPM T-ICHIKAWA
 * @brief PCL6046パラレルバスアクセスソフトウェア&G9006アクセスソフトウェア
 * @version 0.1
 * @date 2023-04-26
 *
 * @copyright Copyright (c) 2023
 *
 */

/* PCLアクセス方式はZ80のフルアドレス方式 */
#include "stdafx.h"
#include "main.h"
#include "pcl_access.h"
#include "g9006_access.h"

/**
 * @brief メインループ
 * 
 */
void main(void) {

    const PIO pio = pio0;                                                                                                    /* PIO0を使用する */
    const uint sm1 = 1;                                                                                                      /* PCLパラレスアクセス用ステートマシン */
	const uint sm2 = 2;                                                                                                      /* G9006_SPIアクセス用ステートマシン*/

    stdio_init_all();                                                                                                        /* To use USB */
    sleep_ms(3000);		                                                                                                     /* wait serial connection */
    
    pcl_bus_init( pio, sm1, IFB, PIO_OUT_BASE, PIO_OUT_NUM, PIO_SIDE_BASE, PIO_SIDE_BASE_NUM, PIO_IN_BASE );                 /* PCLパラレルバス初期化 */
    spi_init( pio, sm2, SPI_MOSI, SPI_MOSI_NUM, SPI_MISO, SPI_MISO_NUM, SPI_SIDE_BASE, SPI_SIDE_BASE_NUM, SPI_SCK_FRQ);	     /* G9006 SPIバス初期化 */
    
    pcl_reg_set( pio, sm1, X_SEL );                                                                                          /* PCL 各種レジスタ設定(X軸) */
    pcl_reg_set( pio, sm1, Y_SEL );                                                                                          /* PCL 各種レジスタ設定(Y軸) */
    
    g9006_init_set( pio, sm2);                                                                                               /* G9006 各種レジスタ設定 */

    org_run(pio, sm1, X_SEL);                                                                                                /* X軸原点位置移動 */
    org_run(pio, sm1, Y_SEL);                                                                                                /* Y軸原点位置移動 */
	
    for( ;; ){
        rcun_update(pio, sm1, sm2);                                                                                          /* PCLのカウンタ値をポート書き込み */
        port_change_chk( pio, sm1, sm2 );                                                                                    /* 仮想デバイス2(デバイス番号2)のポート1,ポート3に変化があったら,PRMVを更新 */
    }
}

/**
 * @brief センサー原点位置移動
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param axs 移動する軸を指定
 */
void org_run(PIO pio, uint sm, uint axs){
    
    printf("%s\n", "PRMD");                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, axs, PRMD, 0x0000001D);               /* -方向に原点サーチモードを設定 */
    read_reg_pcl(pio, sm, IFB, axs, PRMD);                             /* PCLレジスタ読み出し */
    
    write_com_pcl(pio, sm, IFB, axs, COMB, STAFL);                     /* STAFLで動作スタート */

    printf("%s\n", "PRMD");                                            /* UARTで送信 */
    write_reg_pcl( pio, sm, IFB, axs, PRMD, 0x00000042);               /* カウンタ1絶対位置指定モードに設定 */
    read_reg_pcl(pio, sm, IFB, axs, PRMD);                             /* PCLレジスタ読み出し */

}

/**
 * @brief G9006初期設定レジスタ
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 */
void g9006_init_set(PIO pio, uint sm){

    write_reg_G9006( pio, sm, REGWRITE, RADD, 0x4700);        /* RSV=2局(仮想デバイス2つ追加) PMD=7(RCUN用デバイス番号0,1を入力,PRMV用デバイス番号2を出力に設定) */
    sleep_us(100);
    
    read_reg_G9006( pio, sm, REGREAD, RADD);                  /* RADDレジスタを読み出し */
    sleep_us(100);

    write_reg_G9006( pio, sm, PORTCHGSETWRI, 0x00, 0x0A00);   /* ポート変化割り込み設定書き込み(デバイス番号2のポート3,ポート1を変化割り込みに設定) */
    sleep_us(100);

    write_opecom_G9006( pio, sm, OPECOM, CDVON);              /* Motionnet接続 */
    sleep_us(100);
}

/**
 * @brief PCL初期設定レジスタ
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param axs 書き込む軸を指定
 */
void pcl_reg_set(PIO pio, uint sm, uint axs){

    const uint32_t reg_write_data[] = { 0x00000032, 0x00000190, 0x000005AA, 0x00000000, 0x0000012B,     /* PRFL, PRFH, PRUR, PRDR,  PRMG   PRFL=50PPS, PRFH=400PPS, PRMG=1倍*/
                                        0x00000000, 0x00000000, 0x00000064, 0x0000000A, 0x70000000,     /* PRUS, PRDS, RFA,  PRMV,  PRIP */
                                        0x00000064, 0x00000000, 0x00000042, 0x80000000, 0x00100000 };   /* PRCI, PRDP, PRMD, RENV1, RENV3 */
    
    const uint32_t reg_address[] = { PRFL, PRFH, PRUR, PRDR,  PRMG,
                                     PRUS, PRDS, RFA,  PRMV,  PRIP,
                                     PRCI, PRDP, PRMD, RENV1, RENV3 };

    const char *reg_name[] = { "PRFL", "PRFH", "PRUR", "PRDR",  "PRMG",
                               "PRUS", "PRDS", "RFA",  "PRMV",  "PRIP",
                               "PRCI", "PRDP", "PRMD", "RENV1", "RENV3" };

    for(int i = 0; i < 15; i++){
        printf("%s\n", reg_name[i]);                                             /* UARTで送信 */
        write_reg_pcl( pio, sm, IFB, axs, reg_address[i], reg_write_data[i] );   /* PCLレジスタ書き込み */
        read_reg_pcl(pio, sm, IFB, axs, reg_address[i] );                        /* PCLレジスタ読み出し */
    }

    printf("MSTS\n");                                                       /* メインステータス読み出し */
    read_msts_pcl(pio, sm, IFB, axs, MSTS);

}

/**
 * @brief RCUN1カウンタ値読み出し
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm1 PCLパラレルアクセスに使用するステートマシン番号
 * @param sm2 G9006 SPIアクセスに使用するステートマシン番号
 */
void rcun_update(PIO pio, uint sm1, uint sm2){

    printf("RCUN1_XSEL\n");
    uint32_t reg_read_data_6046 = read_reg_pcl(pio, sm1, IFB, X_SEL, RCUN1);  /* X軸RCUN1レジスタ読み出し */
    uint32_t reg_read_data_MSB = (reg_read_data_6046 >> 16) & 0x0000FFFF;     /* RCUNレジスタMSBバイト */
    uint32_t reg_read_data_LSB = reg_read_data_6046 & 0x0000FFFF;             /* RCUNレジスタLSBバイト */

    write_reg_G9006( pio, sm2, PORTWRITE, 0x00, reg_read_data_LSB);           /* X軸RCUN1レジスタLSBバイトをデバイス番号0のポート0,1に書き込み */
    write_reg_G9006( pio, sm2, PORTWRITE, 0x02, reg_read_data_MSB);           /* X軸RCUN1レジスタMSBバイトをデバイス番号0のポート2,3に書き込み */

    printf("RCUN1_YSEL\n");
    reg_read_data_6046 = read_reg_pcl(pio, sm1, IFB, Y_SEL, RCUN1);           /* Y軸RCUN1レジスタ読み出し */
    reg_read_data_MSB = (reg_read_data_6046 >> 16) & 0x0000FFFF;              /* RCUNレジスタMSBバイト */
    reg_read_data_LSB = reg_read_data_6046 & 0x0000FFFF;                      /* RCUNレジスタLSBバイト */

    write_reg_G9006( pio, sm2, PORTWRITE, 0x04, reg_read_data_LSB);           /* Y軸RCUN1レジスタLSBバイトをデバイス番号1のポート0,1に書き込み */
    write_reg_G9006( pio, sm2, PORTWRITE, 0x06, reg_read_data_MSB);           /* Y軸RCUN1レジスタLSBバイトをデバイス番号1のポート2,3に書き込み */

}

/**
 * @brief ポート変化割り込みフラグ確認
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm1 PCLパラレルアクセスに使用するステートマシン番号
 * @param sm2 G9006 SPIアクセスに使用するステートマシン番号
 */
void port_change_chk(PIO pio, uint sm1, uint sm2){
    
    uint32_t port_flg_data = read_reg_G9006(pio, sm2, PORTCHGFLGGET, PORTFLGADDRESS);    /* ポート変化割り込みフラグ読み出し */
    uint port3_flg = (port_flg_data >> 3) & 0x00000001;                                  /* デバイス番号2のポート3ポート変化割り込みフラグ */
    uint port1_flg = (port_flg_data >> 1) & 0x00000001;                                  /* デバイス番号2のポート1ポート変化割り込みフラグ */

    if( port1_flg == 1){
        port_rmv_update(pio, sm1, sm2, 0x08, X_SEL);                                     /* X軸PRMVレジスタの値を更新 */
        port_change_flgclr(pio, sm2, true);                                              /* デバイス番号2のポート1ポート変化割り込みフラグをクリア */
        port1_flg = 0;
    }

    if( port3_flg == 1){
        port_rmv_update(pio, sm1, sm2, 0x0A, Y_SEL);                                     /* Y軸PRMVレジスタの値を更新 */
        port_change_flgclr(pio, sm2, false);                                             /* デバイス番号2のポート3ポート変化割り込みフラグをクリア */
        port3_flg = 0;
    }

}

/**
 * @brief PRMVレジスタ更新
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm1 PCLパラレルアクセスに使用するステートマシン番号
 * @param sm2 G9006 SPIアクセスに使用するステートマシン番号
 * @param address ポート読出しメモリアドレスを指定(08h=デバイス番号2 ポート0,ポート1を読み出し 0Ah=デバイス番号2 ポート2,ポート3を読み出し)
 * @param axs 読み出す軸を指定
 */
void port_rmv_update(PIO pio, uint sm1, uint sm2, uint32_t address, uint axs){

    uint32_t reg_read_data_G9006 = read_reg_G9006( pio, sm2, PORTREAD, address);       /* ポートデータを読み出し */
    uint32_t port_data = (reg_read_data_G9006 << 8) | (reg_read_data_G9006 >> 8);      /* ポートの番号の大きい方が上位に来るようにまとめる */
    uint32_t reg_write_data = 0;
    
    if((port_data & 0x4000) == 0 ){
        reg_write_data = port_data & 0x00003FFF;                                       /* マイナスで無い場合 */
    }else{
        reg_write_data = port_data | 0xFFFF8000;                                       /* マイナスの場合は1で拡張 */
    }

    write_reg_pcl(pio, sm1, IFB, axs, PRMV, reg_write_data);                           /* PRMVレジスタに書き込み */
    write_com_pcl(pio, sm1, IFB, axs, COMB, STAUD);                                    /* STAUDで動作スタート */
}

/**
 * @brief ポート変化割り込みフラグクリア
 * 
 * @param pio 使用するPIO番号(PIO0 or PIO1)
 * @param sm 使用するステートマシン番号(0～3)
 * @param port フラグクリアするポートを指定(true=ポート1のフラグをクリア false=ポート3のフラグをクリア)
 */
void port_change_flgclr(PIO pio, uint sm, bool port){

    if( port ){
        write_reg_G9006(pio, sm, PORTCHGFLGCLR, PORTFLGADDRESS, 0x0200);               /* ポート1のフラグをクリア */
        read_reg_G9006(pio, sm, PORTCHGFLGGET, PORTFLGADDRESS);                        /* フラグを読み出し */
    }else{
        write_reg_G9006(pio, sm, PORTCHGFLGCLR, PORTFLGADDRESS, 0x0800);               /* ポート3のフラグをクリア */
        read_reg_G9006(pio, sm, PORTCHGFLGGET, PORTFLGADDRESS);                        /* フラグを読み出し */
    }
}